% Arezki, Ramey, Sheng Model, "News Shocks in Open Economies: Evidence from Giant Oil Discoveries" forthcoming QJE


% Two Sector Variation on Jaimovich-Rebelo (JR) Open Economy Model, JMCB 2007

% Exploration of results for an economy with the same factor shares across sectors

% Requires: resdepletionghh.xlsx, which gives the baseline models endogenous reserves process

% GHH preferences, Exogenous reserves, same shares in both sectors, diminishing returns (for determinancy)
% The exogenous process for reserves follows the same path followed by 
%  the endogenous path of reserves from the baseline model

% The algebra of steady-states with diminishing returns is complicated
% We were trying to match the depletion case, so we used the y1, y2 values
% as starting values and used other steady-state relationships

% Hints for finding steady-state for different parameters: Move the parameters in the new
% direction until it can't find a SS.  Go back one step to where it could
% and use those values for the starting values and starting searching again


% This program runs using Dynare 4.3.2, but not necessarily on slightly different versions.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% VARIABLE DEFINITIONS

%   y = output y1, y2 for each sector
%   c = consumption
%   i = investment, i1, i2 for each sector
%   di = i - i(-1),  di1, di2 for each sector
%   k = capital stock, k1, k2 for each sector
%   n = employment, n1, n2 for each sector
%   dn = n - n(-1), dn1, dn2 for each sector
%   r = interest rate
%   m = expression in parenthesis in utility function
%   b = net foreign assets 
%   a1 = Hicks neutral technology, a2
%   tb = trade balance 
%   ca = current account 
%   a1, a2  TFP
%   lambda = Lagrange multiplier on net foreign asset accumulation equation
%   eta = Lagrange multiplier on capital accumulation equation, miu in NJ program
%   zeta = Lagrange multiplier on reserve accumulation equation


%  ENDOGENOUS VARIABLES 

var y, y1, y2, c, i, i1, i2, di1, di2, k1, k2, n, n1, n2, dn1, dn2,  m, b,
  r, tb, ca, sv, cy, iy, tby, cay, svy, lambda, eta1, eta2, res, a1, a2; 

%  EXOGENOUS VARIABLES

varexo e;  


% PARAMETERS

parameters alpha1, alpha2, alpha1k, alpha2k, alphar, beta, theta, psi, sigma, gamma, gammai, gammai2, delta, gamman, chi, bstar, poil, a1_ss, a2_ss, res_ss; // structural parameters

% PARAMETER VALUES AND DEFINITIONS


%%%% PRODUCTION FUNCTION PARAMETERS

alpha1 = 0.58;   // exponent on labor in production function
alpha1k = 0.32; 
alpha2 = 0.58;  // exponent on labor in sector 2 - US fraction of VA, also Gross-Hansen
alpha2k = 0.32;   // exponent on capital in sector 2 - Gross-Hansen
alphar = 1 - 0.13 - 0.49;

a1_ss = 1.523;  // Steady-state value of tfp in sector 1
a2_ss = 0.584; //0.384; //0.38; //0.76;  // Steady-state value of tfp in sector 2 - to make sector smaller
res_ss = 6.032338246; //1 //Steady-state value of reserves for endogenous depletion model

%%%% UTILITY FUNCTION PARAMETERS

beta = 1/1.06;  // Discount factor - annual version of JR, similar to average of 7.5% for our countries 
theta = 1.2;  // exponent on labor in utility function, governs elasticity of labor supply to wage, JR value
sigma = 1;  //inverse of the intertemporal substitution of consumption-labor bundle in utility function, sigma = 1 implies ln
ntarget = 20; // target for steady-state n
%          psi = 0.31; //0.239; // gives s-s n around 20 (alternative to ntarget)
gamma = 0; // JR use 0.0001.  JR preference parameter.  gamma = 1 implies KPR preferences, gamma = 0 implies GHH preferences

%%%%% DEPRECIATION, ADJUSTMENT COST, INTEREST ELASTICITY

gammai = 0.1; // 2nd derivative of investment adjustment cost function for i1.  JR quarterly is 1.3, annual should be much less than quarterly
gammai2 = 0.1; // 2nd deriv of investment adjust cost for i2
delta = 0.1;  //  10% is standard for annual capital depreciation rate, JR's is quarterly = 0.0125, annual = 5%
gamman = 0; // JR quarterly is 2. 2nd derivative of labor adjustment cost function

chi =   0.0001; // ** JR value is 0.00001, but this is too low to resolve indeterminancy

poil = 1; // relative price of oil

%%%%% STEADY-STATE RELATIONSHIPS

% These are updated for diminishing returns in sector 1 and 2;

y1_ss = 31.4;
y2_ss = 2;
y_ss = y1_ss + y2_ss;

k1_ss = y1_ss*beta*alpha1k/(1-beta*(1-delta));
k2_ss = y2_ss*beta*alpha2k/(1-beta*(1-delta));

n1_ss = (y1_ss/(a1_ss*k1_ss^alpha1k))^(1/alpha1);
n2_ss = (y2_ss/(a2_ss*k2_ss^alpha2k*res_ss^alphar))^(1/alpha2);

w_ss = alpha1*a1_ss*n1_ss^(alpha1 - 1)*k1_ss^alpha1k;

psi = w_ss/(theta*ntarget^(theta-1)); // parameter to target steady-state n

y_ss = y1_ss + poil*y2_ss;
k_ss = k1_ss + k2_ss;
tby_ss = 0.04;  //Steady-state tb/y , JR value
c_ss = (1 - tby_ss - delta*k_ss/y_ss)*y_ss;
m_ss = c_ss - psi*ntarget^theta;

bstar = -tby_ss*y_ss/(1/beta - 1);  //parameter in interest rate function, set to match tby_ss

tb_ss = tby_ss*y_ss;
b_ss = bstar;
lambda_ss = (c_ss - psi*ntarget^theta)^(-sigma);


model;  

% Note that assumed adj. cost functions are (gammai/2)*(i/i(-1) - 1)^2 for investment
%     and (gamman/2)*(n/n(-1) - 1)^2 for labor

% 1) DEFINITION OF m (variable in parenthesis of utility function)

                m = (c-psi*n^theta);                                          

% 2) FOC C
             m^(-sigma) = lambda;

% 3) and 4) and 5) DEFINITION OF N AND DNs

              n = n1 + n2;

              dn1 = n1/n1(-1);

              dn2 = n2/n2(-1);



% 6) and 7)  FOC N1 AND N2

              m^(-sigma)*psi*theta*n^(theta-1) 
              = alpha1*lambda*y1/n1
             - lambda*(gamman/2)*(dn1-1)^2
             - lambda*dn1*gamman*(dn1-1)
             + beta*lambda(+1)*(dn1(+1))^2*gamman*(dn1(+1)-1) ;


              m^(-sigma)*psi*theta*n^(theta-1)
             = alpha2*y2/n2*poil*lambda
             - lambda*(gamman/2)*(dn2-1)^2
             - lambda*dn2*gamman*(dn2-1)
             + beta*lambda(+1)*(dn2(+1))^2*gamman*(dn2(+1)-1) ;
 

% 8) and 9) K LAW OF MOTION

              k1 = (1-delta)*k1(-1)+ i1*(1 - (gammai/2)*(di1-1)^2);

              k2 = (1-delta)*k2(-1)+ i2*(1 - (gammai2/2)*(di2-1)^2);
             
% 10) and 11) FOC K

               eta1 = beta*eta1(+1)*(1-delta)
             + beta*lambda(+1)*alpha1k*y1(+1)/k1;

               eta2 = beta*eta2(+1)*(1-delta)
             + beta*alpha2k*(y2(+1)/k2)*poil*lambda(+1);


% 12) NET FOREIGN ASSETS LAW OF MOTION

            b = (1 + r)*b(-1) + tb;  


% 13)  FOC NET FOREIGN ASSETS B (B for bonds)

        lambda = beta*lambda(+1)*(1 + r(+1));  


% 14) and 15) DEFINITION OF DI

           di1 = i1/i1(-1);

           di2 = i2/i2(-1);

% 16) and 17) FOC I

            lambda = eta1*(1 - (gammai/2)*(di1-1)^2)
           - eta1*di1*gammai*(di1-1)
           + beta*eta1(+1)*di1(+1)^2*gammai*(di1(+1)-1);

            lambda = eta2*(1 - (gammai/2)*(di2-1)^2)
           - eta2*di2*gammai2*(di2-1)
           + beta*eta2(+1)*di2(+1)^2*gammai2*(di2(+1)-1);   

% 18) RELATION OF R TO NET FOREIGN ASSETS

            r = 1/beta - 1 + chi*( exp( bstar - b(-1)) - 1); 


%  19) and 20) and 21) DEFINITION OF OUTPUT

          y1 = a1*n1^alpha1*(k1(-1)^alpha1k); 

          y2 = a2*n2^alpha2*res(-1)^alphar*k2(-1)^alpha2k; 

          y = y1 + poil*y2;


%  22) and 23) TRADE BALANCE DEFINITIONS

          tb = y - c - i1 - i2
             - n1*(gamman/2)*(dn1-1)^2 - n2*(gamman/2)*(dn2-1)^2;

          tby = tb/y;  

% 24) and 25) CURRENT ACCOUNT DEFINITIONS

          ca = tb + r*b(-1);  // JR timing
          cay = ca/y; // ratio to output 

% 26) Consumption ratio

       cy = c/y;   

% 27) and 28) Aggregate Investment and ratio

      i = i1 + i2;

       iy = i/y;   

% 29) & 30) Saving Definitions

          sv = ca + i1 + i2;
          svy = sv/y;  //  the irf for svy is the same as the above one. 


% 31) Reserves

    res = res_ss + e;

%  32) and 33) TFP

     a1 = a1_ss;

     a2 = a2_ss; 


end;

initval; //from 2sectorghhss.tsp


y      =	y_ss;
y1     =	y1_ss;
y2     =	y2_ss;
c      	=	 c_ss;
i      	=	 delta*k_ss;
i1     	=	 delta*k1_ss;
i2     	=	 delta*k2_ss;
di1    	=	 1;
di2    	=	 1;
k1     	=	 k1_ss;
k2     	=	 k2_ss;
n      	=	 ntarget;
n1     	=	 n1_ss;
n2     	=	 n2_ss;
dn1    	=	 1;
dn2    	=	 1;
m      	=	 m_ss;
b      	=	 bstar;
r      	=	 0.06;
tb     	=	 tby_ss*y_ss;
ca     	=	 0;
sv     	=	 4.31249;
cy     	=	 0.766609;
iy     	=	 0.2;
tby    	=	 0.0333914;
cay    	=	 8.23676e-17;
svy    	=	 0.2;
lambda 	=	 lambda_ss;
eta1   	=	 lambda_ss;
eta2   	=	 lambda_ss;
res =          res_ss;
a1   =  		 a1_ss;
a2   =  		 a2_ss;
end;

check;
steady;

resfile = 'resdepletionghh.xlsx';
resdep = xlsread(resfile);

shocks;
var e;
periods 1:100;
values (resdep);  //Deviation of endogenous reserves from S-S, from arsdepletionghh
end;

simul(periods=100);

%% To print out results (with lots of significant figures) type the following in matlab
%% A=[oo_.exo_simul, oo_.endo_simul']
%% Then double-click A matrix in upper right to get it to print it with significant figures
%%  and then copy to Excel